@@ -533,7 +533,7 @@ abstract class OrderDocument {
 		// EDI
 		if ( $this->is_enabled( 'xml' ) && wpo_ips_edi_is_available() ) {
 			wpo_ips_edi_save_order_taxes( $order );
-			wpo_ips_edi_maybe_save_order_customer_peppol_data( $order );
+			wpo_ips_edi_maybe_save_order_peppol_data( $order );
 		}
 
 		$note = $refund_id ? sprintf(
@@ -1267,8 +1267,12 @@ abstract class OrderDocument {
 		if ( $attachment_id > 0 ) {
 			$company         = $this->get_shop_name();
 			$attachment_src  = wp_get_attachment_image_url( $attachment_id, 'full' );
-			$attachment_path = wp_normalize_path( realpath( get_attached_file( $attachment_id ) ) );
-			$src             = apply_filters( 'wpo_wcpdf_use_path', true ) ? $attachment_path : $attachment_src;
+			$attachment_file = get_attached_file( $attachment_id );
+			$attachment_path = $attachment_file ? wp_normalize_path( realpath( $attachment_file ) ) : '';
+
+			$use_path = apply_filters( 'wpo_wcpdf_use_path', true );
+
+			$src = ( $use_path && ! empty( $attachment_path ) ) ? $attachment_path : $attachment_src;
 
 			if ( empty( $src ) ) {
 				wcpdf_log_error( 'Header logo file not found.', 'critical' );
@@ -1276,7 +1280,7 @@ abstract class OrderDocument {
 			}
 
 			// fix URLs using path
-			if ( ! apply_filters( 'wpo_wcpdf_use_path', true ) && false !== strpos( $src, 'http' ) && false !== strpos( $src, WP_CONTENT_DIR ) ) {
+			if ( ! $use_path && false !== strpos( $src, 'http' ) && false !== strpos( $src, WP_CONTENT_DIR ) ) {
 				$path = preg_replace( '/^https?:\/\//', '', $src ); // removes http(s)://
 				$src  = str_replace( trailingslashit( WP_CONTENT_DIR ), trailingslashit( WP_CONTENT_URL ), $path ); // replaces path with URL
 			}
@@ -1286,8 +1290,16 @@ abstract class OrderDocument {
 				return;
 			}
 
-			$img_src     = isset( WPO_WCPDF()->settings->debug_settings['embed_images'] ) ? wpo_wcpdf_get_image_src_in_base64( $src ) : $src;
-			$img_element = sprintf( '<img src="%1$s" alt="%2$s"/>', wpo_wcpdf_escape_url_path_or_base64( $img_src ), esc_attr( $company ) );
+			$img_src     = isset( WPO_WCPDF()->settings->debug_settings['embed_images'] )
+				? wpo_wcpdf_get_image_src_in_base64( $src )
+				: $src;
+
+			$img_element = sprintf(
+				'<img src="%1$s" alt="%2$s"/>',
+				wpo_wcpdf_escape_url_path_or_base64( $img_src ),
+				esc_attr( $company )
+			);
+
 			$img_element = apply_filters( 'wpo_wcpdf_header_logo_img_element', $img_element, $attachment_id, $this );
 
 			echo $img_element; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
@@ -1626,7 +1638,7 @@ abstract class OrderDocument {
 		$args = $args + $default_args;
 
 		$html = $this->render_template( $this->locate_template_file( "{$this->type}.php" ), array(
-				'order' => $this->order,
+				'order'    => $this->order,
 				'order_id' => $this->order_id,
 			)
 		);
@@ -1690,11 +1702,20 @@ abstract class OrderDocument {
 
 		wpo_ips_edi_file_headers( $quoted, $size );
 
-		ob_clean();
-		flush();
+		// Disable output compression.
+		@ini_set( 'zlib.output_compression', 'Off' );
+
+		// Clear all output buffers to prevent stray bytes/newlines before the XML declaration.
+		while ( ob_get_level() ) {
+			ob_end_clean();
+		}
 
 		if ( WPO_WCPDF()->file_system->exists( $filename_or_contents ) ) {
-			echo WPO_WCPDF()->file_system->get_contents( $filename_or_contents ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
+			$sent = WPO_WCPDF()->file_system->output_file( $filename_or_contents );
+			if ( false === $sent ) {
+				wcpdf_log_error( sprintf( 'Could not output XML file (%s)', $filename_or_contents ), 'critical' );
+			}
+
 			wp_delete_file( $filename_or_contents );
 		}
 
