@@ -4,7 +4,7 @@
  * Requires Plugins:     woocommerce
  * Plugin URI:           https://wpovernight.com/downloads/woocommerce-pdf-invoices-packing-slips-bundle/
  * Description:          Create, print & email PDF or Electronic Invoices & PDF Packing Slips for WooCommerce orders.
- * Version:              5.1.0
+ * Version:              5.5.0
  * Author:               WP Overnight
  * Author URI:           https://www.wpovernight.com
  * License:              GPLv2 or later
@@ -22,7 +22,7 @@ if ( ! class_exists( 'WPO_WCPDF' ) ) :
 
 class WPO_WCPDF {
 
-	public $version              = '5.1.0';
+	public $version              = '5.5.0';
 	public $version_php          = '7.4';
 	public $version_woo          = '3.3';
 	public $version_wp           = '4.4';
@@ -176,6 +176,9 @@ class WPO_WCPDF {
 		$this->frontend            = \WPO\IPS\Frontend::instance();
 		$this->install             = \WPO\IPS\Install::instance();
 		$this->font_synchronizer   = \WPO\IPS\FontSynchronizer::instance();
+
+		// EDI classes
+		\WPO\IPS\EDI\Peppol::instance();
 	}
 
 	/**
@@ -635,8 +638,17 @@ class WPO_WCPDF {
 		$dismiss_arg    = 'wpo_wcpdf_dismiss_unstable_option_announcement';
 		$nonce_action   = 'wcpdf_dismiss_unstable_option_announcement';
 
+		// Fallback if wc_string_to_bool() is unavailable
+		$dismiss_value = get_option( $dismiss_option, 'no' );
+		if ( function_exists( 'wc_string_to_bool' ) ) {
+			$already_dismissed = wc_string_to_bool( (string) $dismiss_value );
+		} else {
+			// simple string check as a fallback
+			$already_dismissed = ( 'yes' === (string) $dismiss_value );
+		}
+
 		// Bail if already dismissed or user cannot manage settings
-		if ( wc_string_to_bool( get_option( $dismiss_option ) ) || ! $this->settings->user_can_manage_settings() ) {
+		if ( $already_dismissed || ! $this->settings->user_can_manage_settings() ) {
 			return;
 		}
 
